/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.document;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.bescheid.attacheditem.AttachedItemTestFactory;

class DocumentMapperTest {

	private DocumentMapper mapper = Mappers.getMapper(DocumentMapper.class);

	@DisplayName("To grpc document")
	@Nested
	class TestToGrpcDocument {

		@Test
		void shouldMapDocument() {
			var result = mapper.toGrpcDocument(DocumentTestFactory.create());

			assertThat(result).usingRecursiveComparison().isEqualTo(GrpcDocumentTestFactory.create());
		}

		@Test
		void shouldMapEmptyDocument() {
			var result = mapper.toGrpcDocument(Document.builder().build());

			assertThat(result).isEqualTo(GrpcDocument.newBuilder().build());
		}
	}

	@DisplayName("From attached item")
	@Nested
	class TestFromAttachedItem {

		@Test
		void shouldMapId() {
			var document = AttachedItemTestFactory.createDocument();

			var result = mapper.fromAttachedItem(document);

			assertThat(result.getId()).isEqualTo(AttachedItemTestFactory.ID);
		}

		@Test
		void shouldMapType() {
			var document = AttachedItemTestFactory.createDocument();

			var result = mapper.fromAttachedItem(document);

			assertThat(result.getType()).isEqualTo(DocumentService.DOCUMENT_TYPE);
		}

		@Test
		void shouldMapFileId() {
			var document = AttachedItemTestFactory.createDocument();

			var result = mapper.fromAttachedItem(document);

			assertThat(result.getFileId()).isEqualTo(DocumentTestFactory.DOCUMENT_FILE);
		}

		@Test
		void shouldMapNachrichtText() {
			var document = AttachedItemTestFactory.createDocument();

			var result = mapper.fromAttachedItem(document);

			assertThat(result.getNachrichtText()).isEqualTo(DocumentTestFactory.NACHRICHT_TEXT);
		}

		@Test
		void shouldMapNachrichtSubject() {
			var document = AttachedItemTestFactory.createDocument();

			var result = mapper.fromAttachedItem(document);

			assertThat(result.getNachrichtSubject()).isEqualTo(DocumentTestFactory.NACHRICHT_SUBJECT);
		}
	}
}