package de.ozgcloud.vorgang.common.migration;

import static de.ozgcloud.vorgang.common.migration.M002_UpdateClientNameInGridFs.*;

import java.util.UUID;

import org.apache.http.entity.ContentType;
import org.bson.Document;
import org.bson.types.ObjectId;

import com.thedeanda.lorem.LoremIpsum;

public class RenameTestFactory {

	// VorgangAttachedItem
	public static final String ITEM_NAME_KEY = "itemName";
	public static final String ITEM_NAME_VALUE = "DummyItemName";
	public static final String ITEM_KEY = "item";
	public static final String ITEM_ATTRIBUTE_KEY = "DummyKey";
	public static final String ITEM_ATTRIBUTE_VALUE = "DummyValue";

	public static Document createVorgangAttachedItem(String clientName) {
		var vorgangAttachedItem = new Document();
		vorgangAttachedItem.put(RenameUtil.CLIENT_KEY, clientName);
		vorgangAttachedItem.put(ITEM_NAME_KEY, ITEM_NAME_VALUE);
		vorgangAttachedItem.put(ITEM_KEY, createItem());

		return vorgangAttachedItem;
	}

	private static Document createItem() {
		var item = new Document();
		item.put(ITEM_ATTRIBUTE_KEY, ITEM_ATTRIBUTE_VALUE);

		return item;
	}

	// GridFsFiles
	public static final String LENGTH_KEY = "length";
	public static final long LENGTH_VALUE = 2L;

	public static final String CHUNK_SIZE_KEY = "chunkSize";
	public static final int CHUNK_SIZE_VALUE = 261120;

	public static final String UPLOAD_DATE_KEY = "uploadDate";
	public static final String UPLOAD_DATE_VALUE = "2023-03-28T07:13:14.821Z";

	public static final String VORGANG_ID_KEY = "vorgangId";
	public static final String VORGANG_ID_VALUE = UUID.randomUUID().toString();

	public static final String FIELD_NAME_KEY = "fieldName";
	public static final String FIELD_NAME_VALUE = LoremIpsum.getInstance().getFirstName();

	public static final String CONTENT_TYPE_KEY = "contentType";
	public static final String CONTENT_TYPE_VALUE = ContentType.APPLICATION_JSON.toString();

	public static final String NAME_KEY = "name";
	public static final String NAME_VALUE = LoremIpsum.getInstance().getFirstName();

	public static final String FILE_NAME_ID_VALUE = new ObjectId().toHexString();
	public static final String FILE_NAME_VALUE = String.format("%s/%s/%s/%s", FILE_NAME_ID_VALUE, OLD_CLIENT_NAME,
			FIELD_NAME_VALUE, NAME_VALUE);

	public static Document createGridFsFile(String clientName) {
		var file = new Document();
		file.put(RenameUtil.METADATA_KEY, createGridFsFileMetadata(clientName));
		file.put(RenameUtil.FILENAME_KEY, FILE_NAME_VALUE);
		file.put(LENGTH_KEY, LENGTH_VALUE);
		file.put(CHUNK_SIZE_KEY, CHUNK_SIZE_VALUE);
		file.put(UPLOAD_DATE_KEY, UPLOAD_DATE_VALUE);

		return file;
	}

	private static Document createGridFsFileMetadata(String clientName) {
		var metadata = new Document();
		metadata.put(VORGANG_ID_KEY, VORGANG_ID_VALUE);
		metadata.put(RenameUtil.CLIENT_KEY, clientName);
		metadata.put(FIELD_NAME_KEY, FIELD_NAME_VALUE);
		metadata.put(CONTENT_TYPE_KEY, CONTENT_TYPE_VALUE);
		metadata.put(NAME_KEY, NAME_VALUE);

		return metadata;
	}

	// Vorgang ClientAttributes
	public final static String DUMMY_ATTRIBUTE_NAME = "dummyAttributeName";

	public final static String CLIENT_ATTRIBUTE_ACCESS_KEY = "access";
	public final static String CLIENT_ATTRIBUTE_ACCESS_VALUE = "READ_ONLY";
	public final static String CLIENT_ATTRIBUTE_VALUE_KEY = "value";
	public final static String CLIENT_ATTRIBUTE_VALUE_VALUE = "2023-11-06";
	public final static String CLIENT_ATTRIBUTE_CLASS_VALUE = "_class";
	public final static String CLIENT_ATTRIBUTE_CLASS_KEY = "ClientAttribute";

	public static Document createVorgangWithClientAttributes(String clientName) {
		var vorgang = new Document();
		vorgang.put(RenameUtil.CLIENT_ATTRIBUTES_KEY, createClientAttribute(clientName));
		return vorgang;
	}

	private static Document createClientAttribute(String clientName) {
		var clientAttribute = new Document();
		clientAttribute.put(clientName, createClientAttributeByClient());
		return clientAttribute;
	}

	private static Document createClientAttributeByClient() {
		var attributeValue = new Document();
		attributeValue.put(CLIENT_ATTRIBUTE_ACCESS_KEY, CLIENT_ATTRIBUTE_ACCESS_VALUE);
		attributeValue.put(CLIENT_ATTRIBUTE_VALUE_KEY, CLIENT_ATTRIBUTE_VALUE_VALUE);
		attributeValue.put(CLIENT_ATTRIBUTE_CLASS_KEY, CLIENT_ATTRIBUTE_CLASS_VALUE);

		var clientAttributeByClient = new Document();
		clientAttributeByClient.put(DUMMY_ATTRIBUTE_NAME, attributeValue);
		return clientAttributeByClient;
	}
}
